/* Evqueue.java */

package eduni.simjava;

import java.util.Iterator;
import java.util.TreeSet;

/**
 * This class implements the event queue used by <code>Sim_system</code>.
 * <p>
 * The event queue is separated into two subqueues, the future queue and
 * the deferred queue. Newly arrived events are added to the future queue
 * while events that the receiving entity is unable to process are placed
 * in the deferred queue. The current implementation uses a TreeSet to store
 * the events received.
 * <p>
 * This class is used internally by <code>Sim_system</code> and should not be
 * directly accessed.
 * @see         eduni.simjava.Sim_system
 * @version     0.1, 25 June 1995
 * @author      Ross McNab
 * 
 * @author      Marcos Dias de Assuncao (Changed this class to use TreeSet)
 */
class FutureEvqueue {
	private TreeSet<Sim_event> sortedSet;
	private long serial = 0;

	// Constructors
	/**
	 * Allocates a new Evqueue object.
	 */
	public FutureEvqueue() {
		sortedSet = new TreeSet<Sim_event>();
	}

	/**
	 * Add a new event to the queue. Adding a new event to the queue 
	 * preserves the temporal order of the events in the queue.
	 * @param new_event	The event to be put in the queue.
	 */
	public void add_event(Sim_event new_event) {
		// The event has to be inserted as the last of all events
	    // with the same event_time(). Yes, this matters.
		new_event.setSerial(serial++);
		sortedSet.add(new_event);
	}
	
	/**
	 * Returns an interator to the queue.
	 * @return the iterator
	 */
	public Iterator<Sim_event> iterator() {
		return sortedSet.iterator();
	}
	
	/**
	 * Returns the size of this event queue
	 * @return the size
	 */
	public int size() {
		return sortedSet.size();
	}
	
	/**
	 * Clears the queue.
	 */
	public void clear(){
		sortedSet.clear();
	}
}
